@@ -1,49 +1,41 @@
 =head1 NAME
 
-Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166-2 codes
+Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166-2 codes, get all states in a country
 
 =head1 SYNOPSIS
 
-    my $country_code = 'GB';
-    my $UK = Locale::SubCountry->new($country_code);
-    if ( not $UK )
+    use Locale::SubCountry;
+    
+    my $fr = Locale::SubCountry->new('France');
+    if ( not $fr )
     {
-        die "Invalid code $country_code\n";
-    }
-    elsif (  $UK->has_sub_countries )
-    {
-        print($UK->full_name('DGY'),"\n");           # Dumfries and Galloway
-        print($UK->regional_division('DGY'),"\n");   # SCT (Scotland)
-    }
-
-    my $australia = Locale::SubCountry->new('Australia');
-    if ( not $australia )
-    {
-        die "Invalid code: Australia\n";
+        die "Invalid country or code: France\n";
     }
     else
     {
-        print($australia->country,"\n");        # Australia
-        print($australia->country_code,"\n");   # AU
+        print($fr->country,"\n");        # France
+        print($fr->country_code,"\n");   # FR
+        print($fr->country_number,"\n"); # 250
 
-        if (  $australia->has_sub_countries )
+        if (  $fr->has_sub_countries )
         {
-            print($australia->code('New South Wales '),"\n");     # NSW
-            print($australia->full_name('S.A.'),"\n");            # South Australia
+            print($fr->code('Hautes-Alpes '),"\n");       # 05
+            print($fr->full_name('03'),"\n");             # Allier
             my $upper_case = 1;
-            print($australia->full_name('Qld',$upper_case),"\n"); # QUEENSLAND
-            print($australia->category('NSW'),"\n");              # state
-            print($australia->FIPS10_4_code('ACT'),"\n");         # 01
-            print($australia->ISO3166_2_code('02'),"\n");         # NSW
+            print($fr->full_name('02',$upper_case),"\n"); # AINSE
+            print($fr->level('02'),"\n");                 # Metropolitan department
+            print($fr->level('A'),"\n");                  # Metropolitan region
+            print($fr->level('BL'),"\n");                 # Overseas territorial collectivity
+            print($fr->levels,"\n");                      # Metropolitan region => 22, Metropolitan department => 96 ...
+ 
+            my @fr_names  = $fr->all_full_names;    # Ain, Ainse, Allier...
+            my @fr_codes   = $fr->all_codes;        # 01, 02, 03 ...
+            my %fr_names_keyed_by_code  = $fr->code_full_name_hash;  # 01 => Ain...
+            my %fr_codes_keyed_by_name  = $fr->full_name_code_hash;  # Ain => 01 ...
 
-            my @aus_state_names  = $australia->all_full_names;
-            my @aus_code_names   = $australia->all_codes;
-            my %aus_states_keyed_by_code  = $australia->code_full_name_hash;
-            my %aus_states_keyed_by_name  = $australia->full_name_code_hash;
-
-            foreach my $code ( sort keys %aus_states_keyed_by_code )
+            foreach my $code ( sort keys %fr_names_keyed_by_code )
             {
-               printf("%-3s : %s\n",$code,$aus_states_keyed_by_code{$code});
+               printf("%-3s : %s\n",$code,$fr_names_keyed_by_code{$code});            
             }
         }
     }
@@ -60,13 +52,17 @@ Locale::SubCountry - Convert state, province, county etc. names to/from ISO 3166
 
 =head1 DESCRIPTION
 
-This module allows you to convert the full name for a countries administrative
-region to the code commonly used for postal addressing. The reverse lookup
+This module allows you to convert the full name for a country's administrative
+region to the code commonly used for postal addressing. The reverse look up
 can also be done.
 
+Lists of sub country codes are useful for web applications that require a valid
+state, county etc to be entered as part of a users location.
+
 Sub countries are termed as states in the US and Australia, provinces
 in Canada and counties in the UK and Ireland. Other terms include region,
-department, city and territory.
+department, city and territory. Countries such as France have several
+levels of sub countries, such as Metropolitan department, Metropolitan region etc.
 
 Names and ISO 3166-2 codes for all sub countries in a country can be
 returned as either a hash or an array.
@@ -78,14 +74,12 @@ fetch every sub country from every country (see examples/demo.pl).
 Sub country codes are defined in "ISO 3166-2,
 Codes for the representation of names of countries and their subdivisions".
 
-ISO 3166-2 codes can be converted to FIPS 10-4 codes. The reverse lookup can
-also be done. Note that FIPS 10-4 has been deprecated.
 
 =head1 METHODS
 
 Note that the following methods duplicate some of the functionality of the
 Locale::Country module (part of the Locale::Codes bundle). They are provided
-here because you may need to first access the list of available countries and
+here because you may need to first access the list of  countries and
 ISO 3166-1 codes, before fetching their sub country data. If you only need
 access to country data, then Locale::Country should be used.
 
@@ -124,7 +118,7 @@ sorted alphabetically.
 
 =head2 all_codes (for world objects)
 
-Given a world object, returns an array of all country IS) 3166-1 codes,
+Given a world object, returns an array of all country ISO 3166-1 codes,
 sorted alphabetically.
 
 
@@ -134,7 +128,7 @@ The C<new> method creates an instance of a sub country object. This must be
 called before any of the following methods are invoked. The method takes a
 single argument, the name of the country that contains the sub country
 that you want to work with. It may be specified either by the ISO 3166-1
-two letter code or the full name. For example:
+alpha-2  code or the full name. For example:
 
     AF - Afghanistan
     AL - Albania
@@ -146,98 +140,82 @@ two letter code or the full name. For example:
     AT - Austria
 
 
-If the code is specified, such as 'AU'  the format may be in  capitals or lower case
+If the code is specified, such as 'AU'  the format may be in capitals or lower case
 If the full name is specified, such as 'Australia', the format must be in title case
 If a country name or code is specified that the module doesn't recognised, it will issue a warning.
 
 =head2 country
 
-Returns the current country of a sub country object, the format is title case
+Returns the current country name of a sub country object. The format is in title case,
+such as 'United Kingdom'
 
 =head2 country_code
 
-Given a sub country object, returns the two letter ISO 3166-1 code of the country
+Given a sub country object, returns the alpha-2 ISO 3166-1 code of the country,
+such as 'GB'
 
 
 =head2 code
 
 Given a sub country object, the C<code> method takes the full name of a sub
-country and returns the sub country's ISO 3166-2 code. The full name can appear
+country and returns the sub country's alpha-2 ISO 3166-2 code. The full name can appear
 in mixed case. All white space and non alphabetic characters are ignored, except
 the single space used to separate sub country names such as "New South Wales".
 The code is returned as a capitalised string, or "unknown" if no match is found.
 
 =head2 full_name
 
-Given a sub country object, the C<full_name> method takes the ISO 3166-2 code of
-a sub country and returns the sub country's full name. The code can appear
+Given a sub country object, the C<full_name> method takes the alpha-2 ISO 3166-2 code
+of a sub country and returns the sub country's full name. The code can appear
 in mixed case. All white space and non alphabetic characters are ignored. The
 full name is returned as a title cased string, such as "South Australia".
 
 If an optional argument is supplied and set to a true value, the full name is
 returned as an upper cased string.
 
-=head2 category
+=head2 level
 
-Given a sub country object, the C<category> method takes the ISO 3166-2 code of
-a sub country and returns the sub country's category type. Examples are city,
-province,state and district. The category is returned as a capitalised string,
-or "unknown" if no match is found.
+Given a sub country object, the C<level> method takes the alpha-2 ISO 3166-2 code
+of a sub country and returns the sub country's level . Examples are city,
+province,state and district, and usually relates to the a regions size.
+The level is returned as a  string, or "unknown" if no match is found.
 
-=head2 regional_division
-
-Given a sub country object, the C<regional_division> method takes the
-ISO 3166-2 code of a sub country and returns the sub country's
-regional_division. This is an alphanumeric code. The regional_division
-is returned as a capitalised string,  or "unknown" if no match is found.
 
 =head2 has_sub_countries
 
 Given a sub country object, the C<has_sub_countries> method returns 1 if the
 current country has sub countries, or 0 if it does not. Some small countries
-such as Singapore do not have sub countries.
-
-=head2 FIPS10_4_code
-
-NOTE: On September 2, 2008, FIPS 10-4 was one of ten standards withdrawn by NIST as a
-Federal Information Processing Standard. It may not be supported by this module in
-the future.
-
-Given a sub country object, the C<FIPS_10_4_code> method takes the ISO 3166-2 code
-of a sub country and returns the sub country's FIPS 10-4 code, or the string 'unknown',
-if none exists. FIPS is a standard  developed by the US government.
-
-=head2 ISO3166_2_code
-
-Given a sub country object, the C<ISO3166_2_code> method takes the FIPS 10-4 code
-of a sub country and returns the sub country's ISO 3166-2 code, or the string 'unknown',
-if none exists.
+such as New Caledonia" do not have sub countries.
 
 
-=head2 full_name_code_hash  (for subcountry objects)
+=head2 full_name_code_hash  (for sub country objects)
 
 Given a sub country object, returns a hash of all full name/code pairs,
 keyed by sub country name. If the country has no sub countries, returns undef.
 
-=head2 code_full_name_hash  (for subcountry objects)
+=head2 code_full_name_hash  (for sub country objects)
 
 Given a sub country object, returns a hash of all code/full name pairs,
 keyed by sub country code. If the country has no sub countries, returns undef.
 
 
-=head2 all_full_names  (for subcountry objects)
+=head2 all_full_names  (for sub country objects)
 
 Given a sub country object, returns an array of all sub country full names,
 sorted alphabetically. If the country has no sub countries, returns undef.
 
-=head2 all_codes  (for subcountry objects)
+=head2 all_codes  (for sub country objects)
 
-Given a sub country object, returns an array of all sub country ISO 3166-2 codes,
-sorted alphabetically. If the country has no sub countries, returns undef.
+Given a sub country object, returns an array of all sub country alpha-2 ISO 3166-2 codes.
+If the country has no sub countries, returns undef.
 
 
 =head1 SEE ALSO
 
+All codes have been downloaded from the latest version of the Alioth project
+L<https://pkg-isocodes.alioth.debian.org/>
+
+
 L<Locale::Country>,L<Lingua::EN::AddressParse>,
 L<Geo::StreetAddress::US>,L<Geo::PostalAddress>,L<Geo::IP>
 L<WWW::Scraper::Wikipedia::ISO3166> for obtaining ISO 3166-2 data
@@ -264,49 +242,43 @@ the correct code for that sub country. These entries are usually duplicated
 because the name represents two different types of sub country, such as a
 province and a geographical unit. Examples are:
 
-    AZERBAIJAN : Lankaran; LA (the City), LAN (the Rayon) [see note]
+    AZERBAIJAN : Lankaran; LA (the Municipality), LAN (the Rayon) [see note]
     AZERBAIJAN : Saki; SA,SAK [see note]
     AZERBAIJAN : Susa; SS,SUS
     AZERBAIJAN : Yevlax; YE,YEV
-    INDONESIA  : Kalimantan Timur; KI,KT
-    LAOS       : Vientiane VI,VT
-    MOLDOVA    : Hahul; CA,CHL
-    MOLDOVA    : Bubasari; DU,DBI
-    MOLDOVA    : Hrhei; OR,OHI
-    MOLDOVA    : Coroca; SO,SOA
-    MOLDOVA    : Gngheni; UN,UGI
-    MOZAMBIQUE : Maputo; MPM,L
+    LAOS       : Vientiane VI the Vientiane, VT the Prefecture
+    MOZAMBIQUE : Maputo; MPM (City),L (Province)
 
-Note: these names are spelt with a diaeresis character (two dots) above
-some of the 'a' characters. This causes utf8 errors on some versions
-of Perl, so they are omitted here. See the Locale::SubCountry::Data module
+Note: these names are spelt with  diacrtic characters (such as two dots above
+some of the 'a' characters). This causes utf8 errors on some versions
+of Perl, so they are omitted here. See the Locale::SubCountry::Codes module
 for correct spelling
 
-FIPS codes are not provided for all sub countries.
-
 
 =head1 AUTHOR
 
 Locale::SubCountry was written by Kim Ryan <kimryan at cpan dot org>.
 
+=head1 COPYRIGHT AND LICENCE
+
+This software is Copyright (c) 2016 by Kim Ryan.
+
+This library is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
+
 
 =head1 CREDITS
 
 Ron Savage for many corrections to the data
 
-Alastair McKinstry provided many of the sub country codes and names.
 
 Terrence Brannon produced Locale::US, which was the starting point for
 this module.
 
-Mark Summerfield and Guy Fraser provided the list of UK counties.
-
-TJ Mather supplied the FIPS codes and many amendments to the sub country data
 
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2015 Kim Ryan. All rights reserved.
+Copyright (c) 2016 Kim Ryan. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
@@ -319,16 +291,17 @@ use strict;
 use warnings;
 use locale;
 use Exporter;
-use Locale::SubCountry::Data;
+use JSON;
+use Locale::SubCountry::Codes;
 
 #-------------------------------------------------------------------------------
 
 package Locale::SubCountry::World;
-our $VERSION = '1.66';
+our $VERSION = '2.01';
 
 # Define all the methods for the 'world' class here. Note that because the
 # name space inherits from the Locale::SubCountry name space, the
-# package wide variables $SubCountry::country_lookup and $Locale::SubCountry::subcountry_lookup are
+# package wide variables $SubCountry::country and $Locale::SubCountry::subcountry are
 # accessible.
 
 
@@ -350,7 +323,7 @@ sub new
 sub code_full_name_hash
 {
     my $world = shift;
-    return(  %{ $Locale::SubCountry::country_lookup{_code_keyed} } );
+    return(  %{ $Locale::SubCountry::country{_code_keyed} } );
 }
 #-------------------------------------------------------------------------------
 # Returns a hash of name/code pairs for all countries, keyed by country name.
@@ -358,7 +331,7 @@ sub code_full_name_hash
 sub full_name_code_hash
 {
     my $world = shift;
-    return( %{ $Locale::SubCountry::country_lookup{_full_name_keyed} } );
+    return( %{ $Locale::SubCountry::country{_full_name_keyed} } );
 }
 #-------------------------------------------------------------------------------
 # Returns sorted array of all country full names
@@ -366,7 +339,7 @@ sub full_name_code_hash
 sub all_full_names
 {
     my $world = shift;
-    return ( sort keys %{ $Locale::SubCountry::country_lookup{_full_name_keyed} });
+    return ( sort keys %{ $Locale::SubCountry::country{_full_name_keyed} });
 }
 #-------------------------------------------------------------------------------
 # Returns sorted array of all two letter country codes
@@ -374,137 +347,62 @@ sub all_full_names
 sub all_codes
 {
     my $world = shift;
-    return ( sort keys %{ $Locale::SubCountry::country_lookup{_code_keyed} });
+    return ( sort keys %{ $Locale::SubCountry::country{_code_keyed} });
 }
 
 #-------------------------------------------------------------------------------
 
 package Locale::SubCountry;
-our $VERSION = '1.66';
+our $VERSION = '2.01';
 
 #-------------------------------------------------------------------------------
-# Initialization code must be run first to create global data structure.
+# Initialization code which will be run first to create global data structure.
 # Read in the list of abbreviations and full names defined in the
-# Locale::SubCountry::Data package
+# Locale::SubCountry::Codes package
 
 {
 
-    unless ( $Locale::SubCountry::Data::xml_data )
+    unless ( $Locale::SubCountry::Codes::JSON )
     {
-      die "Could not locate Locale::SubCountry::Data::xml_data variable";
+      die "Could not locate Locale::SubCountry::Codes::JSON variable";
     }
 
-    # Get all the data from the Locale::SubCountryData pakage and place into an array of lines
-    my @lines = split(/\n/,$Locale::SubCountry::Data::xml_data);
+    # Get all the data from the Locale::SubCountry::Codes package and place into a structure
 
-    while ( @lines )
-    {
-        my $current_line = shift(@lines);
+    # Note: will fail on badly formed JSON data
+    my $json_text = $Locale::SubCountry::Codes::JSON;
+    my $json = JSON->new->allow_nonref;
+    
+    my $all_codes_ref = $json->decode($json_text);
+    
+        
+    foreach my $country_ref ( @{ $all_codes_ref->{'3166-1'} })
+    {             
+        # Create doubly indexed hash, keyed by country code and full name.
+        # The user can supply either form to create a new sub_country
+        # object, and the objects properties will hold both the countries
+        # name and it's code.
 
-        # Data is in XML format, use a simple parser to extract it
-        my ($country_name,$country_code);
-        if ( $current_line =~ /<country>/ )
-        {
-            # start of a  <country> .. </country> block
-            my $country_finished = 0;
-            until ( $country_finished )
-            {
-                $current_line = shift(@lines);
-                if ( $current_line =~ /\s*<name>(.*)<\/name>/ )
-                {
-                    $country_name = $1;
-                }
-                elsif ( $current_line =~ /\s*<code>(.*)<\/code>/ )
-                {
-                    $country_code = $1;
-                }
-                elsif ( $current_line =~ /<subcountry>/ )
-                {
-                    my $sub_country_finished = 0;
+        $Locale::SubCountry::country{_code_keyed}{$country_ref->{alpha_2}} = $country_ref->{name};
+        $Locale::SubCountry::country{_full_name_keyed}{$country_ref->{name}} = $country_ref->{alpha_2};
+        
+        # Get numeric code for country, such as Australia = '036'
+        $Locale::SubCountry::country{$country_ref->{name}}{_numeric }= $country_ref->{numeric};
+    }
 
-                    my ($sub_country_name,$sub_country_code,$category,$regional_division,$FIPS_code);
-                    until ( $sub_country_finished )
-                    {
-                        $current_line = shift(@lines);
-                        if ( $current_line =~ /\s*<name>(.*)<\/name>/ )
-                        {
-                            $sub_country_name = $1;
-                        }
-                        elsif ( $current_line =~ /\s*<code>(.*)<\/code>/ )
-                        {
-                            $sub_country_code = $1;
-                        }
-                        elsif ( $current_line =~ /\s*<FIPS>(.*)<\/FIPS>/ )
-                        {
-                            $FIPS_code = $1;
-                        }
-                        elsif ( $current_line =~ /\s*<category>(.*)<\/category>/ )
-                        {
-                            $category = $1;
-                        }
-                        elsif ( $current_line =~ /\s*<regional_division>(.*)<\/regional_division>/ )
-                        {
-                            $regional_division = $1;
-                        }
-                        elsif ( $current_line =~ /<\/subcountry>/ )
-                        {
-
-                            $sub_country_finished = 1;
-
-                            # Some sub countries have no ISO code, such as Shariff Kabunsuan in the
-                            # Phillipines. Only index sub country if it has a code
-                            if ( defined $sub_country_code )
-                            {
-                                # Insert into doubly indexed hash, grouped by country for ISO 3166-2
-                                # codes. One hash is keyed by abbreviation and one by full name. Although
-                                # data is duplicated, this provides the fastest lookup and simplest code.
-
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{_code_keyed}{$sub_country_code} = $sub_country_name;
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{_full_name_keyed}{$sub_country_name} = $sub_country_code;
-                            }
-
-                            if ( $category )
-                            {
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{$sub_country_code}{_category} = $category;
-                            }
-
-                            if ( $regional_division )
-                            {
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{$sub_country_code}{_regional_division} = $regional_division;
-                            }
-
-                            if ( $FIPS_code )
-                            {
-                                # Insert into doubly indexed hash, grouped by country for FIPS 10-4 codes
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{_FIPS10_4_code_keyed}{$FIPS_code} = $sub_country_code;
-                                $Locale::SubCountry::subcountry_lookup{$country_name}{_ISO3166_2_code_keyed}{$sub_country_code} = $FIPS_code;
-                            }
-                        }
-                        elsif ( $current_line =~ /\w.*/ )
-                        {
-                            die "Badly formed XML sub country data in $country_name\nData: $current_line\n";
-                        }
-                    }
-                }
-                elsif ( $current_line =~ /<\/country>/ )
-                {
-                    $country_finished = 1;
-
-                    # Create doubly indexed hash, keyed by  country code and full name.
-                    # The user can supply either form to create a new sub_country
-                    # object, and the objects properties will hold both the countries
-                    # name and it's code.
-
-                    $Locale::SubCountry::country_lookup{_code_keyed}{$country_code} = $country_name;
-                    $Locale::SubCountry::country_lookup{_full_name_keyed}{$country_name} = $country_code;
-
-                }
-                elsif ( $current_line =~ /\w.*/ )
-                {
-                    die "Badly formed XML country data in $country_name\nData: $current_line\n";
-                }
-            }
-        }
+     
+    foreach my $sub_country_ref ( @{ $all_codes_ref->{'3166-2'} })
+    {    
+        my ($country_code,$sub_country_code) = split(/\-/,$sub_country_ref->{code});
+        my $sub_country_name =  $sub_country_ref->{name};
+         
+        $Locale::SubCountry::subcountry{$country_code}{_code_keyed}{$sub_country_code} = $sub_country_name;
+        $Locale::SubCountry::subcountry{$country_code}{_full_name_keyed}{$sub_country_name} = $sub_country_code;        
+        $Locale::SubCountry::subcountry{$country_code}{$sub_country_code}{_level} = $sub_country_ref->{type};
+        
+        # Record  level occurence in a country
+        $Locale::SubCountry::subcountry{$country_code}{_levels}{$sub_country_ref->{type}}++; 
+       
     }
 }
 
@@ -516,19 +414,17 @@ sub new
     my $class = shift;
     my ($country_or_code) = @_;
 
-
     my ($country,$country_code);
 
-
     # Country may be supplied either as a two letter code, or the full name
     if ( length($country_or_code) == 2 )
     {
         $country_or_code = uc($country_or_code); # lower case codes may be used, so fold to upper case
-        if ( $Locale::SubCountry::country_lookup{_code_keyed}{$country_or_code} )
+        if ( $Locale::SubCountry::country{_code_keyed}{$country_or_code} )
         {
             $country_code = $country_or_code;
             # set country to it's full name
-            $country = $Locale::SubCountry::country_lookup{_code_keyed}{$country_code};
+            $country = $Locale::SubCountry::country{_code_keyed}{$country_code};
          }
         else
         {
@@ -538,10 +434,10 @@ sub new
     }
     else
     {
-        if ( $Locale::SubCountry::country_lookup{_full_name_keyed}{$country_or_code} )
+        if ( $Locale::SubCountry::country{_full_name_keyed}{$country_or_code} )
         {
             $country = $country_or_code;
-            $country_code = $Locale::SubCountry::country_lookup{_full_name_keyed}{$country};
+            $country_code = $Locale::SubCountry::country{_full_name_keyed}{$country};
         }
         else
         {
@@ -555,13 +451,14 @@ sub new
     bless($sub_country,$class);
     $sub_country->{_country} = $country;
     $sub_country->{_country_code} = $country_code;
+    $sub_country->{_numeric} = $Locale::SubCountry::country{$country}{_numeric};
 
 
     return($sub_country);
 }
 
 #-------------------------------------------------------------------------------
-# Returns the current country of the sub country object
+# Returns the current country's name of the sub country object
 
 sub country
 {
@@ -569,7 +466,7 @@ sub country
     return( $sub_country->{_country} );
 }
 #-------------------------------------------------------------------------------
-# Returns the current country code of the sub country object
+# Returns the current country's alpha2 code of the sub country object
 
 sub country_code
 {
@@ -577,6 +474,15 @@ sub country_code
     return( $sub_country->{_country_code} );
 }
 
+#-------------------------------------------------------------------------------
+# Returns the current country's numeric code of the sub country object
+
+sub country_number
+{
+    my $sub_country = shift;
+    return( $sub_country->{_numeric} );
+}
+
 #-------------------------------------------------------------------------------
 # Given the full name for a sub country, return the ISO 3166-2 code
 
@@ -595,7 +501,7 @@ sub code
 
     $full_name = _clean($full_name);
 
-    my $code = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_full_name_keyed}{$full_name};
+    my $code = $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_full_name_keyed}{$full_name};
 
     # If a code wasn't found, it could be because the user's capitalization
     # does not match the one in the look up data of this module. For example,
@@ -614,7 +520,7 @@ sub code
         {
             if ( uc($full_name) eq uc($current_name) )
             {
-                $code = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_full_name_keyed}{$current_name};
+                $code = $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_full_name_keyed}{$current_name};
             }
         }
     }
@@ -628,96 +534,9 @@ sub code
         return('unknown');
     }
 }
-#-------------------------------------------------------------------------------
-# Given the ISO 3166-2 code for a sub country, return the FIPS 104-4 code.
-
-sub FIPS10_4_code
-{
-    my $sub_country = shift;
-    my ($code) = @_;
-
-    $code = _clean($code);
-    $code = uc($code);
-
-    my $FIPS_code = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_ISO3166_2_code_keyed}{$code};
-
-    if ( $FIPS_code )
-    {
-        return($FIPS_code);
-    }
-    else
-    {
-        return('unknown');
-    }
-}
-#-------------------------------------------------------------------------------
-# Given the FIPS 10-4 code for a sub country, return the ISO 3166-2 code.
-
-sub ISO3166_2_code
-{
-    my $sub_country = shift;
-    my ($FIPS_code) = @_;
-
-    $FIPS_code = _clean($FIPS_code);
-
-    my $code = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_FIPS10_4_code_keyed}{$FIPS_code};
-
-    if ( $code )
-    {
-        return($code);
-    }
-    else
-    {
-        return('unknown');
-    }
-}
-#-------------------------------------------------------------------------------
-# Given the ISO 3166-2 code for a sub country, return the category,
-# being state, province, city, council etc
-
-sub category
-{
-    my $sub_country = shift;
-    my ($code) = @_;
-
-    $code = _clean($code);
-
-    my $category = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{$code}{_category};
-
-    if ( $category )
-    {
-        return($category);
-    }
-    else
-    {
-        return('unknown');
-    }
-}
 
 #-------------------------------------------------------------------------------
-# Given the ISO 3166-2 code for a sub country, return the regional division,
-
-sub regional_division
-{
-    my $sub_country = shift;
-    my ($code) = @_;
-
-    $code = _clean($code);
-
-    my $regional_division = $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{$code}{_regional_division};
-
-    if ( $regional_division )
-    {
-        return($regional_division);
-    }
-    else
-    {
-        return('unknown');
-    }
-}
-
-#-------------------------------------------------------------------------------
-# Given the ISO 3166-2 code for a sub country, return the full name.
+# Given the alpha-2 ISO 3166-2 code for a sub country, return the full name.
 # Parameters are the code and a flag, which if set to true
 # will cause the full name to be uppercased
 
@@ -729,14 +548,14 @@ sub full_name
     unless ( $sub_country->has_sub_countries )
     {
         # this country has no sub countries
-        return;
+        # return;
     }
 
     $code = _clean($code);
     $code = uc($code);
 
     my $full_name =
-        $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_code_keyed}{$code};
+        $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_code_keyed}{$code};
     if ( $uc_name )
     {
         $full_name = uc($full_name);
@@ -751,13 +570,47 @@ sub full_name
         return('unknown');
     }
 }
+
+#-------------------------------------------------------------------------------
+# Given the alpha-2 ISO 3166-2 code for a sub country, return the level,
+# being one of state, province, overseas territory, city, council etc
+sub level
+{
+    my $sub_country = shift;
+    my ($code) = @_;
+
+    $code = _clean($code);
+
+    my $level = $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{$code}{_level};
+
+    if ( $level )
+    {
+        return($level);
+    }
+    else
+    {
+        return('unknown');
+    }
+}
+#-------------------------------------------------------------------------------
+# Given a sub country object, return a hash of all the levels and their totals 
+# Such as Australia: State => 6, Territory => 2
+
+sub levels
+{
+    my $sub_country = shift;  
+ 
+    return( %{ $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_levels} });
+
+}
+
 #-------------------------------------------------------------------------------
 # Returns 1 if the current country has sub countries. otherwise 0.
 
 sub has_sub_countries
 {
     my $sub_country = shift;
-    if ( $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_code_keyed} )
+    if ( $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_code_keyed} )
     {
         return(1);
     }
@@ -774,7 +627,7 @@ sub code_full_name_hash
     my $sub_country = shift;
     if ( $sub_country->has_sub_countries )
     {
-        return( %{ $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_code_keyed} } );
+        return( %{ $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_code_keyed} } );
     }
     else
     {
@@ -789,7 +642,7 @@ sub full_name_code_hash
     my $sub_country = shift;
     if ( $sub_country->has_sub_countries )
     {
-        return( %{ $Locale::SubCountry::subcountry_lookup{$sub_country->{_country}}{_full_name_keyed} } );
+        return( %{ $Locale::SubCountry::subcountry{$sub_country->{_country_code}}{_full_name_keyed} } );
     }
     else
     {
@@ -816,7 +669,7 @@ sub all_full_names
     }
 }
 #-------------------------------------------------------------------------------
-# Returns sorted array of all sub country ISO 3166-2 codes for the current country
+# Returns array of all sub country alpha-2  ISO 3166-2 codes for the current country
 
 sub all_codes
 {
